#!/bin/bash
#
# Author Krigler Pavol
# Version 0.3.3
#
# Bash script Cisco Template Manager (CTM) for checking interface templates
# and directories.
#
# Copyright 2011 Pavol Krigler <pavol.krigler@gelogic.net>
#
# This file is part of Cisco Template Manager.
#
#    CTM is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    CTM is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with CTM.  If not, see <http://www.gnu.org/licenses/>.

# CTM configuration file
CONFIGFILE="/etc/ctm/ctm.conf"

# Constants
AVAILABLE_TEMPLATES=1

# Detect if file contains empty lines
function empty_lines() {
	if [ $(grep -c "^$" $1) -gt 0 ]; then 
		REPORT+="\tfile \"$(basename $1)\" contains empty line(s)\n"
	fi
}

# Find duplicates services between all devices
function find_duplicates() {
	local DUP_NUM=0	

	DEVICES=$(find . -maxdepth 1 -type d -printf "%f\n" | grep -v "^.$")
	for DEVICE in $DEVICES; do
		ROLES=$(find $DEVICE -maxdepth 1 -type d -printf "%f\n" | grep -v "^${DEVICE}$")
		for ROLE in $ROLES; do
			find $DEVICE/$ROLE/services -type f -exec md5sum {} \; >> $HOME/.ctm-check-templates-$$
		done
	done

	DUPLICATES=$(cat $HOME/.ctm-check-templates-$$ | cut -d " " -f 1 | sort | uniq -c | grep -v "      1 " | sed -e 's/........//')

	for ENTRY in $DUPLICATES; do
		DUP_NUM=$(($DUP_NUM+1))
		SERVICE=$(cat $HOME/.ctm-check-templates-$$ | grep -m 1 $ENTRY | cut -d " " -f 3)
		echo "($DUP_NUM) Duplicated template content in Device/Role/Template"
		RESULT=$(cat $HOME/.ctm-check-templates-$$ | grep $ENTRY | cut -d " " -f 3)
		for LINE in $RESULT; do
			echo "$LINE" | cut -d '/' -f 1,2,4
		done
		echo 
	done
	
	# delete temporary file
	rm -f $HOME/.ctm-check-templates-$$
	exit 
}

# print usage
function usage() {
	echo "Usage: $0 report|duplicates"
	echo ""
	echo "mandatory arguments:"
	echo "  report             print template consistency report from all devices and roles"
	echo "  duplicates         show duplicated services between all devices and roles"
	echo "                     it is better to use symbolic links to manage same services"

	exit
}

function print_report() {

	for DEVICE in $(ls -1); do
	# Skip files 
		test ! -d $DEVICE && continue
		ROLES=$(ls -1 $DEVICE)
	
		if [ -z "$ROLES" ]; then
			echo "Device directory \"$DEVICE\" exists but no roles defined"
			continue
		fi
	
		for ROLE in $ROLES; do
			test ! -d $DEVICE/$ROLE && continue
	
			# Check if all directories and files exists
			
			# SECTIONS
			if [ ! -d $DEVICE/$ROLE/sections ]; then
				REPORT+="\tmissing \"sections\" directory\n"
			else
					NUM_OF_SERVICE=$(ls -1 $DEVICE/$ROLE/sections | wc -l)
				if [ $NUM_OF_SERVICE -eq 0 ]; then
					REPORT+="\tno section(s) defined in \"sections\" directory\n"
					fi
			fi 

			# SERVICES
			if [ ! -d $DEVICE/$ROLE/services ]; then
				REPORT+="\tmissing \"services\" directory\n"
			else

				# Check if every line contains start character '^' or EOL '$' character. 
				# If some line(s) does not contains '^' or '$' character the template is markeda as faulty
				SERVICE_FILTER=$(ls -1 $DEVICE/$ROLE/services | grep -v ".exclude$" | grep -v ".model$" | grep -v ".def$")
				for ENTRY in $SERVICE_FILTER; do
					STATUS=$(grep -v '^!' $DEVICE/$ROLE/services/$ENTRY | egrep -v '^\^|\$$')
					test ! -z "$STATUS" && REPORT+="\tmissing \"^\" and/or \"$\" characters in $DEVICE/$ROLE/services/$ENTRY template file\n"
				done
	
				# Definition files			
				# Check if both lines have SERVICE_ID tag definition
				DEF_FILTER=$(ls -1 $DEVICE/$ROLE/services |  grep ".def$" | sed -e 's/.def//')
	                        for ENTRY in $DEF_FILTER; do
					test $(grep "^FORMAT=" -c $DEVICE/$ROLE/services/${ENTRY}.def) -eq 0 && REPORT+="\tmissing \"FORMAT=\" variable in ${ENTRY}.def definition file\n"
					test $(grep "^EXAMPLE=" -c $DEVICE/$ROLE/services/${ENTRY}.def) -eq 0 && REPORT+="\tmissing \"EXAMPLE=\" variable in ${ENTRY}.def definition file\n"
					RESULT=$(grep -c "${ENTRY}" $DEVICE/$ROLE/services/${ENTRY}.def)
					test $RESULT -ne 2 && REPORT+="\tmissing SERVICE_ID tag [${ENTRY}] in definition file ${ENTRY}.def\n"
				done



				TRIPLETS=$(ls -1 $DEVICE/$ROLE/services | grep -v ".exclude$" | sed -e 's/.def$//' | sed -e 's/.model$//' | sort | uniq -c | grep -v '^      3 ' | sed -e 's/........//')

				if [ -z "$(ls -1 $DEVICE/$ROLE/services)" ]; then
						REPORT+="\tno service(s) defined in \"services\" directory"
				else
					for ENTRY in $TRIPLETS; do
						test ! -e $DEVICE/$ROLE/services/$ENTRY && REPORT+="\tmissing template file \"$ENTRY\" in the \"services\" directory\n"
						test ! -e $DEVICE/$ROLE/services/$ENTRY.model && REPORT+="\tmissing model file \"$ENTRY.model\" in the \"services\" directory\n"
						test ! -e $DEVICE/$ROLE/services/$ENTRY.def && REPORT+="\tmissing definition file \"$ENTRY.def\" in the \"services\" directory\n"
					done
				fi
			fi 
	
			# GLOBAL.MODEL
			if [ ! -e $DEVICE/$ROLE/global.model ]; then
				REPORT+="\tmissing main \"global.model\" template file\n"
			else
				empty_lines "$DEVICE/$ROLE/global.model"
			fi 

			# GLOBAL.TEMPLATE
			if [ ! -e $DEVICE/$ROLE/global.template ]; then
				REPORT+="\tmissing main \"global.template\" template file\n"
			else
				empty_lines "$DEVICE/$ROLE/global.template"
			fi 
	
			# GLOBAL.EXCLUDE
			if [ -e $DEVICE/$ROLE/global.exclude ]; then
				empty_lines "$DEVICE/$ROLE/global.exclude"
			fi 
			
			# Print report if some error has occured
			if [ -n "$REPORT" ]; then
			echo Device/Role: $DEVICE/$ROLE
				echo -e $REPORT	
				REPORT=""
			fi
		done
	done
	exit
}

# Read configuration file
if [ ! -s "$CONFIGFILE" ]; then 
	echo "ERROR: CTM configuration file '$CONFIGFILE' not found or is empty"
	exit 1
fi

. $CONFIGFILE

cd $TEMPLATE_DIR

test $# -eq 0 && usage

test $1 = "duplicates" && find_duplicates
test $1 = "report" && print_report

usage

